#include "map_view.hpp"
#include <functional>
#include <string>
#include "dbc.hpp"
#include "components.hpp"
#include "rand.hpp"
#include "animation.hpp"
#include "rand.hpp"
#include <codecvt>
#include <iostream>

namespace gui {
  using namespace components;

  MapViewUI::MapViewUI(GameLevel &level) :
    $level(level), $tiles(level.map->width(), level.map->height())
  {
  }

  void MapViewUI::update_level(GameLevel &level) {
    $level = level;
  }

  void MapViewUI::init(int x, int y, int w, int h) {
    $gui.position(x, y, w, h);
    $gui.layout("[map_grid]");

    auto grid = $gui.entity("map_grid");
    $gui.set<guecs::WideText>(grid, {L"Loading...", 45, ColorValue::DARK_LIGHT, 10});
    $gui.set<guecs::Sprite>(grid, {"paper_ui_background"});

    $gui.init();
  }

  void MapViewUI::render(sf::RenderWindow &window) {
    $tiles = $level.map->tiles();
    auto grid = $gui.entity("map_grid");
    auto player_pos = $level.world->get<Position>($level.player).location;

    std::string map;

    matrix::box it{$level.map->walls(), player_pos.x, player_pos.y, 7, 3};

    while(it.next())
    {
      if(it.x == player_pos.x && it.y == player_pos.y) {
        map += "@";
      } else {
        map += $tiles.at(it.x, it.y).display;
      }

      if(it.x == it.right - 1) map += "\n";
    }

    std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> converter;
    std::wstring map_wstr = converter.from_bytes(map);

    auto& map_text = $gui.get<guecs::WideText>(grid);
    map_text.update(map_wstr);

    $gui.render(window);
  }
}
